# © 2020 Le Filament (<https://www.le-filament.com>)
# © 2020 Confédération Générale des Scop (<https://www.les-scop.coop>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

from odoo import _, api, fields, models
from odoo.exceptions import ValidationError

PERCENT = [
    ("0", "0"),
    ("20", "20"),
    ("40", "40"),
    ("60", "60"),
    ("80", "80"),
    ("100", "100"),
]


class ScopPartner(models.Model):
    _inherit = "res.partner"

    # Processus d'incubation
    incub_status = fields.Selection(
        [
            ("1_prospection", "Phase de prospection et de candidature"),
            ("2_pre-incubation", "Phase de pré-incubation"),
            ("3_incubation", "Phase d'incubation"),
            ("4_sortie_incub", "Sortie d'incubation"),
            ("9_abandonne", "Projet abandonné"),
        ],
        tracking=True,
        string="Statut incubation",
        index=True,
    )
    incub_out_mode = fields.Selection(
        [
            ("1_non_def", "Non définie"),
            ("2_non_coop", "Non coopérative"),
            ("3_coop_non_adh", "Coopérative non adhérente"),
            ("4_coop_adh", "Coopérative projet d'adhésion"),
        ],
        default="1_non_def",
        tracking=True,
        string="Sortie incubation",
        index=True,
    )
    incub_out_done = fields.Boolean(string="Société créée", default=False)
    incub_active = fields.Boolean(string="Projet actif", default=True)
    # Données projet
    incub_prescription_id = fields.Many2one(
        comodel_name="cgscop_incub.prescription",
        string="Canal de prescription",
        tracking=True,
    )
    incub_date_candidature = fields.Date(string="Date de candidature", tracking=True)
    incub_date_prospection = fields.Date(
        string="Date de début du dossier", tracking=True
    )
    incub_delegate_id = fields.Many2one(
        comodel_name="res.users",
        string="Délégué incubation",
        ondelete="restrict",
        tracking=True,
    )
    incub_thematique_ids = fields.Many2many(
        comodel_name="cgscop_incub.thematique",
        column1="partner_id",
        column2="thematique_id",
        string="Thematique du projet",
        tracking=True,
    )
    incub_typologie_porteur_id = fields.Many2one(
        comodel_name="cgscop_incub.typologie_porteur",
        string="Typologie du porteur de projet",
        tracking=True,
    )
    incub_projet_description = fields.Text(
        string="Description du projet",
        tracking=True,
    )
    incub_nb_assos = fields.Integer(
        string="Nombre d'associés au projet",
        tracking=True,
    )
    incub_sensibilite_coop = fields.Selection(
        selection=PERCENT,
        string="Sensibilté coopérative",
        tracking=True,
    )
    incub_percent_realisation = fields.Selection(
        selection=PERCENT,
        string="Probabilité de réalisation",
        tracking=True,
    )
    incub_fiche_entree = fields.Boolean(string="Fiche d'entrée", tracking=True)
    incub_fiche_sortie = fields.Boolean(string="Fiche de sortie", tracking=True)
    incub_convention = fields.Boolean(
        string="Convention d'accompagnement",
        tracking=True,
    )
    incub_preincub_bypass = fields.Boolean(
        string="Pas de pré-incubation",
        tracking=True,
    )
    incub_incubation_bypass = fields.Boolean(string="Pas d'incubation", tracking=True)
    incub_preincub_deb = fields.Date(
        string="Date d'entrée pré-incubation", tracking=True
    )
    incub_preincub_fin = fields.Date(
        string="Date de sortie pré-incubation", tracking=True
    )
    incub_parcours_preincub = fields.Many2one(
        comodel_name="cgscop_incub.parcours_preincub",
        string="Parcours pré-incubation",
        tracking=True,
    )
    incub_statut_prefig = fields.Many2one(
        comodel_name="cgscop_incub.statut_prefig",
        string="Statuts de préfiguration",
        tracking=True,
    )
    incub_prefig_date = fields.Date(
        string="Date de création préfiguration", tracking=True
    )
    incub_inter_bilan_date = fields.Date(
        string="Date de bilan intermédiaire", tracking=True
    )
    incub_incubation_deb = fields.Date(string="Date d'entrée incubation", tracking=True)
    incub_incubation_fin = fields.Date(
        string="Date de sortie incubation", tracking=True
    )
    incub_expertise_ids = fields.One2many(
        comodel_name="cgscop_incub.expertise_projet",
        inverse_name="partner_id",
        string="Expertises",
    )
    incub_preincub_mt_exp = fields.Float(
        string="Montant expertises pre-incub",
        compute="_compute_preincub_montant",
        store=True,
    )
    incub_incubation_mt_exp = fields.Float(
        string="Montant expertises incubation",
        compute="_compute_incubation_montant",
        store=True,
    )
    incub_financement_ids = fields.One2many(
        comodel_name="cgscop_incub.financement_projet",
        inverse_name="partner_id",
        string="Financements",
    )
    incub_financement_mt_dmd = fields.Float(
        string="Financement montant demandé",
        compute="_compute_financement_mt",
        store=True,
    )
    incub_financement_mt_recu = fields.Float(
        string="Financement montant recu",
        compute="_compute_financement_mt",
        store=True,
    )
    incub_sortie_notes = fields.Text(
        string="Commentaires fin de projet",
        tracking=True,
    )
    incub_motif_abandon_id = fields.Many2one(
        "cgscop_incub.motif_abandon",
        string="Motif d'abandon",
        tracking=True,
    )
    incub_out_is_cae = fields.Boolean(
        string="A rejoint une CAE existante",
        tracking=True,
    )
    incub_out_cae_name = fields.Char(string="Nom de la CAE", tracking=True)

    # ------------------------------------------------------
    # Computed
    # ------------------------------------------------------
    @api.depends("incub_expertise_ids")
    def _compute_preincub_montant(self):
        for partner in self:
            xps = partner.incub_expertise_ids.search(
                [
                    ("partner_id", "=", partner.id),
                    ("incub_phase", "=", "2_pre-incubation"),
                ]
            )
            if xps:
                for xp in xps:
                    partner.incub_preincub_mt_exp = (
                        partner.incub_preincub_mt_exp + xp.montant
                    )

    @api.depends("incub_expertise_ids")
    def _compute_incubation_montant(self):
        for partner in self:
            xps = partner.incub_expertise_ids.search(
                [
                    ("partner_id", "=", partner.id),
                    ("incub_phase", "=", "3_incubation"),
                ],
            )
            if xps:
                for xp in xps:
                    partner.incub_incubation_mt_exp = (
                        partner.incub_incubation_mt_exp + xp.montant
                    )

    @api.depends("incub_financement_ids")
    def _compute_financement_mt(self):
        for partner in self:
            fis = partner.incub_financement_ids.search(
                [["partner_id", "=", partner.id]]
            )
            if fis:
                for fi in fis:
                    partner.incub_financement_mt_dmd = (
                        partner.incub_financement_mt_dmd + fi.montant_dmd
                    )
                    partner.incub_financement_mt_recu = (
                        partner.incub_financement_mt_recu + fi.montant_recu
                    )

    # ------------------------------------------------------
    # Constains
    # ------------------------------------------------------
    @api.constrains("incub_preincub_deb", "incub_preincub_fin")
    def _preincub_date_constrains(self):
        for rec in self:
            if rec.incub_preincub_fin and rec.incub_preincub_deb:
                if rec.incub_preincub_fin < rec.incub_preincub_deb:
                    raise ValidationError(
                        _(
                            "La date de fin de pré-incubation \
                        doit être supérieure à la date de début"
                        )
                    )

    @api.constrains("incub_incubation_deb", "incub_incubation_fin")
    def _incubation_date_constrains(self):
        for rec in self:
            if rec.incub_incubation_fin and rec.incub_incubation_deb:
                if rec.incub_incubation_fin < rec.incub_incubation_deb:
                    raise ValidationError(
                        _(
                            "La date de fin d'incubation doit \
                        être supérieure à la date de début"
                        )
                    )

    @api.constrains("incub_preincub_fin", "incub_incubation_deb")
    def _incub_date_constrains(self):
        for rec in self:
            if rec.incub_preincub_fin and rec.incub_incubation_deb:
                if rec.incub_incubation_deb < rec.incub_preincub_fin:
                    raise ValidationError(
                        _(
                            "La date de début d'incubation doit \
                        être supérieure à la date de fin de pre-incubation"
                        )
                    )

    # ------------------------------------------------------
    # Button
    # ------------------------------------------------------
    def incub_non_coop(self):
        for rec in self:
            rec.is_cooperative = False
            rec.incub_out_done = True

    def incub_coop_non_adh(self):
        for rec in self:
            rec.is_cooperative = True
            rec.project_status = "6_suivi"
            rec.incub_out_done = True

    def incub_coop_adh(self):
        for rec in self:
            rec.is_cooperative = True
            rec.project_status = "2_pre-diagnostic"
            rec.incub_out_done = True

    def toggle_incub_active(self):
        for rec in self:
            rec.incub_active = not rec.incub_active
